/**
 *
 *  Betterville Portrait Handler
 *  @author Drakkos
 *  @started 27/03/2009
 *
 */

// This is our class that holds all the data about a portrait.  
// We use this to store all of the configuration details about 
// the wizards on display.
class portrait {
  string name;
  string title;
  string eyes;
  string height;
  string weight;
  string looks;
  string hair;
  string history;
  int secret_panel;
}

// This is our data set of all the portraits in the handler.
class portrait *all_portraits;

void setup_data();

void create() {
  seteuid (geteuid());
  setup_data();
}

/**
 *
 *  This method generates a random name for a wizard, consisting of forename 
 *  and surname.
 *
 *  @returns                A random wizard name
 *
 */
 
string random_wizard_name() {
  string *forename = ({"albert", "mustrum", "ponder", "bill", "achmed",
    "drum", "eric", "eskarina", "harry"});
  string *surname = ({"malicho", "ridcully", "stibbons", "rincewind", 
    "billet", "smith", "dread"});

  return element_of(forename) + " " + element_of(surname);
}  

/**
 *
 *  This method generates a random title for a wizard, consisting of a 
 *  numeric level and an order.
 *
 *  @returns                A random wizard title
 *
 */
 
string random_wizard_title() {
  int level = 4 + random (5);
  string *orders = ({
    "The Ancient and Truly Original Sages of the Unbroken Circle",
    "The Ancient Order of the Dynastic Crescent",
    "The Ancient Order of Djinn Diviners",
    "The Hoodwinkers",
    "The Last Order",
    "Mrs. Widgery's Lodgers",
    "The Order of Midnight",
    "The Ancient Order of the Scintillating Scarab",
    "The Venerable Council of Seers",
    "The Sages of the Unknown Shadow",
    "The Ancient and Truly Original Brothers of the Silver Star"});
  
  return "the " + word_ordinal (level) + " level wizard of " 
    + element_of (orders);
}

/**
 *
 *  This method finds a portrait in the data set.
 *
 *  @param      name          The name of the portrait to find.
 *  @returns                  -1 if the portrait does not exist, and the index 
 *                            of the portrait if it does.
 *
 */
 
int find_portrait (string name) {
  for (int i = 0; i < sizeof (all_portraits); i++) {
    if (all_portraits[i]->name == name) {
      return i;
    }
  }
  return -1;
}

/**
 *
 *  This method generates a random portrait and adds it to the data-set. 
 *
 *  @param      suspected     Whether this is the portrait that may be hiding
 *                            a secret panel.  1 indicates that it is a possibility
 *                            but not the hiding place, and 2 is the hiding place.
 *
 */
 
void add_portrait(int suspected) {
  class portrait new_portrait;
  string wname;
  string wtitle;
  string *eyes_arr = ({"blue", "green", "black", "red"});
  string *height_arr = ({"tall", "short", "medium-height"});
  string *weight_arr = ({"fat", "skinny", "stocky", "plump"});
  string *looks_arr = ({"handsome", "ugly", "grotesque", "beautiful"});
  string *hair_arr = ({"black", "blonde", "grey", "white", "brown"}); 
  string *banal_history = ({
    "he was a great battle wizard in the employ of the dwarfs at Koom Valley.",
    "he was a great archchancellor of Unseen University.",
    "he was thoroughly unremarkable in every way.",
    "he had no distinguishing accomplishments."
 });
 string *suspect_history = ({
    "he was an especially cunning and secretive researcher.",
    "he had a gift for traps, secret doors, and mechanical constructions.",
    "he was known for hiding things in obvious places.",
  });
  string hist;
 
  // The suspected value determines what kind of history goes along with the 
  // portrait.  if it's not-zero, then we give a banal history, otherwise
  // we give a history that hints at secrets. 
  if (suspected) {
    hist = element_of (suspect_history);
  }
  else {
    hist = element_of (banal_history);
  }      
  
  // We want the name to be unique, so we generate random names in a loop
  // until we find one that hasn't been used.  Be careful with this kind 
  // of suspect, because if you generate more names than are possible
  // random choices, it'll cause an infinite loop.
  do {
    wname = random_wizard_name();
  } while (find_portrait (wname) != -1);
  
  wtitle = random_wizard_title();
  
  // Create the portrait instance from the supplied data.
  new_portrait = new (class portrait, 
    name: wname, 
    title: wtitle, 
    eyes: element_of (eyes_arr),
    height: element_of (height_arr),
    weight: element_of (weight_arr),
    looks: element_of (looks_arr),
    hair: element_of (hair_arr),
    history: hist,
    secret_panel: suspected 
  );
  
  // Add the new portrait to the data set.
  all_portraits += ({ new_portrait });
}

/**
 *
 *  Setup the handler with a new set of data.
 *
 */
 
void setup_data() {
  // Generate the index at which the secret panel may be 
  // found.
 int location = random (10);
 int panel;
 
 all_portraits = ({ });
 
 // Create ten portraits,
 for (int i = 0; i < 10; i++) {
  panel = 0;
  
  // If this is the index where we have the secret panel,
  // mark it is so.
  if (i == location) {
    panel = 2;
  }
  // Every third portrait is a suspected hiding place.
  else if (i % 3 == 0) {
    panel = 1;
  }
  
  add_portrait (panel);
 }
}

/**
 *
 *  Get the long description of a specific wizard portrait.
 *
 *  @returns            Returns the properly formatted description of a portrait.
 *
 */
 
string portrait_long (string wizard) {
  int i = find_portrait (wizard);
  
  if ( i == -1) {
    return 0;
  }
 
  // We use sprintf to generate the string - sprintf allows us to 
  // format a string and pass in  the constituent parts as parameters.  
  // %s is essentially a place-holder for a string to follow. 
  return sprintf ("This is a portrait of %s, %s.  He is a %s, %s man, "
    "with %s hair, %s eyes and a %s face.\n", 
    cap_words (all_portraits[i]->name),
    all_portraits[i]->title,
    all_portraits[i]->height,
    all_portraits[i]->weight,
    all_portraits[i]->hair,
    all_portraits[i]->eyes,
    all_portraits[i]->looks);
}

/**
 *
 *  Get the history associated with a particular wizard.
 *
 *  @param      wizard        The name of the wizard to get the history for.
 *  @returns                  The history associated with the wizard.
 *
 */
 
string query_portrait_history (string wizard) {
  int i = find_portrait (wizard);
  
  if ( i == -1) {
    return 0;
  }
  
  return all_portraits[i]->history;
}

/**
 *
 *  Get the value stored for a particular secret_panel value.
 *
 *  @param      wizard        The name of the wizard to get the secret value for.
 *  @returns                  The secret panel value for the wizard.  0 indicates
 *                            no secret, 1 indicates a suspect, and 2 indicates that
 *                            this is the hiding place of the secret panel.
 *
 */
 
int query_portrait_panel (string wizard) {
  int i = find_portrait (wizard);
  
  if ( i == -1) {
    return 0;
  }
  
  return all_portraits[i]->secret_panel;
}

/**
 *
 *  Returns the list of all wizards represented by portraits
 *
 *  @returns                  An array of wizard names.
 *
 */
 
string* query_portraits () {
  string *names = ({ });
  
  for (int i = 0; i < sizeof (all_portraits); i++) {
    names += ({ all_portraits[i]->name });
  }
  
  return names;
}
