#include "path.h"
#include <tasks.h>

inherit INHERITS + "outside_room";

string horse_tail_function();

void setup() {
  set_short ("skeleton room");
  add_property ("determinate", "a ");
   
  set_long ("This is a skeleton room.  There is some brush here.\n");

  add_item ("brush", "It looks unpleasant, full of nettles and "
    "other horrible stinging things.");
  add_item (({"nettle", "horrible stinging thing"}), "You wouldn't "
    "want to take a widdle in that lot.");

  // Add an item for horse-tail, a well known folk rememedy for nettle
  // stings. 
  add_item ("horsetail", (: horse_tail_function :));
    
  set_light (100);

  add_exit ("north", ROOMS + "betterville_03", "road");
  add_exit ("southwest", ROOMS + "betterville_01", "road");
  add_exit ("southeast", ROOMS + "betterville_08", "road");

  modify_exit ("southeast", ({"obvious", 0}));
}

string horse_tail_function() {
  int *effects;
  string ret;
  int val;
    
  ret = "Yes, there is some horsetail here.  ";
  
  // Get the effects on the player that match the provided classification.  
  // Since effects that don't have a merge_effect will be applied 
  // multiple times, we get these back as an array.
  effects = this_player()->effects_matching ("body.nettles");

  // If the array is empty, there is no matching effect.
  if (sizeof (effects)) {
    ret += "Throbbing from the stinging nettles, you clutch it "
      "thankfully."; 
      
      // If there is, we need to step over each to do whatever 
      // we need to do.
    for (int i = 0; i < sizeof (effects); i++) {
      // arg_of gives us the argument of the effect.  For our nettles
      // effect, it's just an integer.
      val = this_player()->arg_of (effects[i]);
      
      // Subtract 25 from the argument.
      val = val - 25;

      // Now, if in doing so we reduce the value to less than 0, 
      // just remove the effect.
      if (val <= 0) {
        // Away you go, vile effect.  delete_effect takes an 
        // effect number (enum) as a parameter, and removes it
        // utterly.
        this_player()->delete_effect (effects[i]);
      }
      else {
        // Otherwise, set the argument of this effect to be the 
        // new value.  
        this_player()->set_arg_of(effects[i], val);
      }
    }  
  }
  
  return ret;
}

/**
 *
 *  do_search gets called on the room whenever the player hits the 
 *  'search' command.
 *
 *  @param        str         Any text provided after the search
 *  @returns                  1 if the search succeeds, 0 if it fails,
 *                            and -1 if it should fail and give the
 *                            default message.
 *
 */
 
int do_search (string str) {
  int found;
  int success;    
  int *effects;
  
  // Check to see if the player has the nettles effect on them. 
  effects = this_player()->effects_matching ("body.nettles");
  
  // If they do, stop them searching properly.
  if (sizeof (effects)) {
    // No no no!
    tell_object (this_player(), "You are still stinging from your brush with "
      "the nettles, and you can't bring yourself to search too effectively.\n");
    return -1;
  }
  
  // Put them through the taskmaster to see if they notice the path leading 
  // off to the southeast.
  success = TASKER->perform_task (this_player(), "adventuring.perception", 100, TM_FREE);

  switch (success) {
    case AWARD:
      tell_object (this_player(), "%^YELLOW%^%^BOLD%^You feel a little more "
        "perceptive.\n%^RESET%^");
    case SUCCEED:      
      tell_object (this_player(), "You see a trail leading through the brush "
        "to the southeast.\n");
      found = 1;
    break;
    case FAIL:
      found = 0;
    break;
    default:
  }

  // There's a 50% change that they will afflict themselves with
  // the nettles effect as a result of searching.
  if (random (2)) {
    // add_effect does just what it says on the tin.  We give the 
    // name of the effect, and the argument we want to apply the 
    // effect with.
    this_player()->add_effect (EFFECTS + "nettles", 100 + random (100));    
  }
  
  if (found) {
    return 1;
  }
  else {
    return -1;
  }
  
}